@@ -1,49 +1,21 @@
 /*
+ * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @license AGPL-3.0
  *
- *    OPEN-XCHANGE legal information
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *    All intellectual property rights in the Software are protected by
- *    international copyright laws.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Affero General Public License for more details.
  *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with OX App Suite.  If not, see <https://www.gnu.org/licenses/agpl-3.0.txt>.
  *
- *    In some countries OX, OX Open-Xchange, open xchange and OXtender
- *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the OX Software GmbH group of companies.
- *    The use of the Logos is not covered by the GNU General Public License.
- *    Instead, you are allowed to use these Logos according to the terms and
- *    conditions of the Creative Commons License, Version 2.5, Attribution,
- *    Non-commercial, ShareAlike, and the interpretation of the term
- *    Non-commercial applicable to the aforementioned license is published
- *    on the web site http://www.open-xchange.com/EN/legal/index.html.
- *
- *    Please make sure that third-party modules and libraries are used
- *    according to their respective licenses.
- *
- *    Any modifications to this package must retain all copyright notices
- *    of the original copyright holder(s) for the original code used.
- *
- *    After any such modifications, the original and derivative code shall remain
- *    under the copyright of the copyright holder(s) and/or original author(s)per
- *    the Attribution and Assignment Agreement that can be located at
- *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
- *    given Attribution for the derivative code and a license granting use.
- *
- *     Copyright (C) 2016-2020 OX Software GmbH
- *     Mail: info@open-xchange.com
- *
- *
- *     This program is free software; you can redistribute it and/or modify it
- *     under the terms of the GNU General Public License, Version 2 as published
- *     by the Free Software Foundation.
- *
- *     This program is distributed in the hope that it will be useful, but
- *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- *     for more details.
- *
- *     You should have received a copy of the GNU General Public License along
- *     with this program; if not, write to the Free Software Foundation, Inc., 59
- *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Any use of the work other than as authorized under this license or copyright law is prohibited.
  *
  */
 
@@ -61,12 +33,12 @@ import com.openexchange.config.lean.LeanConfigurationService;
 import com.openexchange.drive.events.DriveEvent;
 import com.openexchange.drive.events.DriveEventPublisher;
 import com.openexchange.drive.events.apn.APNAccess;
+import com.openexchange.drive.events.apn.APNAccess.AuthType;
 import com.openexchange.drive.events.apn.APNCertificateProvider;
 import com.openexchange.drive.events.apn2.util.ApnsHttp2Options;
 import com.openexchange.drive.events.subscribe.DriveSubscriptionStore;
 import com.openexchange.drive.events.subscribe.Subscription;
 import com.openexchange.exception.OXException;
-import com.openexchange.java.Streams;
 import com.openexchange.java.Strings;
 import com.openexchange.server.ServiceLookup;
 import com.openexchange.threadpool.Task;
@@ -121,13 +93,40 @@ public class APNDriveEventPublisher implements DriveEventPublisher {
         /*
          * get APN options via config cascade if configured
          */
-        String keystore = configService.getProperty(userId, contextId, DriveEventsAPNProperty.keystore, optionals);
-        if (Strings.isNotEmpty(keystore)) {
-            String password = configService.getProperty(userId, contextId, DriveEventsAPNProperty.password, optionals);
-            boolean production = configService.getBooleanProperty(userId, contextId, DriveEventsAPNProperty.production, optionals);
-            String topic = configService.getProperty(userId, contextId, DriveEventsAPNProperty.topic, optionals);
-            LOG.trace("Using configured keystore {}, {} service for push via {} for user {} in context {}.", keystore, production ? "production" : "sandbox", serviceId, I(userId), I(contextId));
-            return new APNAccess(keystore, password, production, topic);
+        String sType = configService.getProperty(userId, contextId, DriveEventsAPNProperty.authtype, optionals);
+        AuthType type = AuthType.authTypeFor(sType);
+        if (null == type) {
+            LOG.debug("Missing or invalid authentication type for push via {} for user {} in context {}. Using fall-back \"certificate\".", serviceId, I(userId), I(contextId));
+            type = AuthType.CERTIFICATE;
+        }
+        switch (type) {
+            case CERTIFICATE:
+                /*
+                 * APN with certificate in keystore is configured
+                 */
+                String keystore = configService.getProperty(userId, contextId, DriveEventsAPNProperty.keystore, optionals);
+                if (Strings.isNotEmpty(keystore)) {
+                    String password = configService.getProperty(userId, contextId, DriveEventsAPNProperty.password, optionals);
+                    boolean production = configService.getBooleanProperty(userId, contextId, DriveEventsAPNProperty.production, optionals);
+                    String topic = configService.getProperty(userId, contextId, DriveEventsAPNProperty.topic, optionals);
+                    LOG.trace("Using configured keystore {}, {} service for push via {} for user {} in context {}.", keystore, production ? "production" : "sandbox", serviceId, I(userId), I(contextId));
+                    return new APNAccess(keystore, password, production, topic);
+                }
+                break;
+            case JWT:
+                /*
+                 * APN with private key is configured
+                 */
+                String privateKey = configService.getProperty(userId, contextId, DriveEventsAPNProperty.privatekey, optionals);
+                if (Strings.isNotEmpty(privateKey)) {
+                    String keyId = configService.getProperty(userId, contextId, DriveEventsAPNProperty.keyid, optionals);
+                    String teamId = configService.getProperty(userId, contextId, DriveEventsAPNProperty.teamid, optionals);
+                    String topic = configService.getProperty(userId, contextId, DriveEventsAPNProperty.topic, optionals);
+                    boolean production = configService.getBooleanProperty(userId, contextId, DriveEventsAPNProperty.production, optionals);
+                    LOG.trace("Using configured private key {}, {} service for push via {} for user {} in context {}.", privateKey, production ? "production" : "sandbox", serviceId, I(userId), I(contextId));
+                    return new APNAccess(privateKey, keyId, teamId, topic, production);
+                }
+                break;
         }
         /*
          * check for a registered APN options provider as fallback, otherwise
@@ -148,8 +147,7 @@ public class APNDriveEventPublisher implements DriveEventPublisher {
          */
         List<Subscription> subscriptions = null;
         try {
-            subscriptions = requireService(DriveSubscriptionStore.class, services).getSubscriptions(
-                event.getContextID(), new String[] { serviceId }, event.getFolderIDs());
+            subscriptions = requireService(DriveSubscriptionStore.class, services).getSubscriptions(event.getContextID(), new String[] { serviceId }, event.getFolderIDs());
         } catch (OXException e) {
             LOG.error("unable to get subscriptions for service {}", serviceId, e);
         }
@@ -188,23 +186,45 @@ public class APNDriveEventPublisher implements DriveEventPublisher {
     }
 
     private ApnsHttp2Options getApn2Options(APNAccess access) {
-        Object store = access.getKeystore();
-        String password = access.getPassword();
-        boolean production = access.isProduction();
-        String topic = access.getTopic();
-        if (store instanceof byte[]) {
-            return new ApnsHttp2Options((byte[]) store, password, production, topic);
-        }
-        if (store instanceof String) {
-            FileInputStream in = null;
-            try {
-                in = new FileInputStream((String) store);
-                byte[] data = IOUtils.toByteArray(in);
-                return new ApnsHttp2Options(data, password, production, topic);
-            } catch (Exception e) {
-                LOG.error("Error loading keystore", e);
-            } finally {
-                Streams.close(in);
+        switch (access.getAuthType()) {
+            case CERTIFICATE: {
+                Object store = access.getKeystore();
+                String password = access.getPassword();
+                boolean production = access.isProduction();
+                String topic = access.getTopic();
+                if (store instanceof byte[]) {
+                    return new ApnsHttp2Options((byte[]) store, password, production, topic, null);
+                }
+                if (!(store instanceof String)) {
+                    break;
+                }
+
+                try (FileInputStream in = new FileInputStream((String) store)) {
+                    return new ApnsHttp2Options(IOUtils.toByteArray(in), password, production, topic, null);
+                } catch (Exception e) {
+                    LOG.error("Error loading keystore", e);
+                }
+                break;
+            }
+            case JWT: {
+                Object privateKey = access.getPrivateKey();
+                String keyId = access.getKeyId();
+                String teamId = access.getTeamId();
+                String topic = access.getTopic();
+                boolean production = access.isProduction();
+                if (privateKey instanceof byte[]) {
+                    return new ApnsHttp2Options((byte[]) privateKey, keyId, teamId, production, topic, null);
+                }
+                if (!(privateKey instanceof String)) {
+                    break;
+                }
+
+                try (FileInputStream in = new FileInputStream((String) privateKey)) {
+                    return new ApnsHttp2Options(IOUtils.toByteArray(in), keyId, teamId, production, topic, null);
+                } catch (Exception e) {
+                    LOG.error("Error loading private key", e);
+                }
+                break;
             }
         }
         return null;
