@@ -1,55 +1,28 @@
 /*
+ * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @license AGPL-3.0
  *
- *    OPEN-XCHANGE legal information
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *    All intellectual property rights in the Software are protected by
- *    international copyright laws.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Affero General Public License for more details.
  *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with OX App Suite.  If not, see <https://www.gnu.org/licenses/agpl-3.0.txt>.
  *
- *    In some countries OX, OX Open-Xchange, open xchange and OXtender
- *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the OX Software GmbH group of companies.
- *    The use of the Logos is not covered by the GNU General Public License.
- *    Instead, you are allowed to use these Logos according to the terms and
- *    conditions of the Creative Commons License, Version 2.5, Attribution,
- *    Non-commercial, ShareAlike, and the interpretation of the term
- *    Non-commercial applicable to the aforementioned license is published
- *    on the web site http://www.open-xchange.com/EN/legal/index.html.
- *
- *    Please make sure that third-party modules and libraries are used
- *    according to their respective licenses.
- *
- *    Any modifications to this package must retain all copyright notices
- *    of the original copyright holder(s) for the original code used.
- *
- *    After any such modifications, the original and derivative code shall remain
- *    under the copyright of the copyright holder(s) and/or original author(s)per
- *    the Attribution and Assignment Agreement that can be located at
- *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
- *    given Attribution for the derivative code and a license granting use.
- *
- *     Copyright (C) 2016-2020 OX Software GmbH
- *     Mail: info@open-xchange.com
- *
- *
- *     This program is free software; you can redistribute it and/or modify it
- *     under the terms of the GNU General Public License, Version 2 as published
- *     by the Free Software Foundation.
- *
- *     This program is distributed in the hope that it will be useful, but
- *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- *     for more details.
- *
- *     You should have received a copy of the GNU General Public License along
- *     with this program; if not, write to the Free Software Foundation, Inc., 59
- *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Any use of the work other than as authorized under this license or copyright law is prohibited.
  *
  */
 
 package com.openexchange.drive.events.apn.osgi;
 
 import java.io.File;
+import java.io.FileInputStream;
 import java.io.IOException;
 import java.util.Collections;
 import java.util.Map;
@@ -168,8 +141,37 @@ public class APNDriveEventsActivator extends HousekeepingActivator {
     protected APNAccess createAccess(Properties properties, OperationSystemType type, FragmentPropertiesLoader loader, String topic) {
         try {
             Map<String, String> optionals = Collections.singletonMap(DriveEventsAPNProperty.OPTIONAL_FIELD, type.getName());
-            String keystore = getProperty(properties, DriveEventsAPNProperty.keystore, optionals);
 
+            // Try with PKCS#8 private key first
+            String privateKey = optProperty(properties, DriveEventsAPNProperty.privatekey, optionals);
+            if (Strings.isNotEmpty(privateKey)) {
+                String keyId = getProperty(properties, DriveEventsAPNProperty.keyid, optionals);
+                String teamId = getProperty(properties, DriveEventsAPNProperty.teamid, optionals);
+                boolean production = Boolean.parseBoolean(getProperty(properties, DriveEventsAPNProperty.production, optionals));
+
+                // Check file path validity
+                if (new File(privateKey).exists()) {
+                    try (FileInputStream keyIn = new FileInputStream(privateKey)) {
+                        return new APNAccess(Streams.stream2bytes(keyIn), keyId, teamId, topic, production);
+                    } catch (IOException e) {
+                        LOG.warn("Could not read file: {}", privateKey, e);
+                    }
+                }
+
+                // Assume file is given as resource identifier
+                try {
+                    byte[] privateKeyBytes = Streams.stream2bytes(loader.loadResource(privateKey));
+                    if (privateKeyBytes.length == 0) {
+                        return null;
+                    }
+                    return new APNAccess(privateKeyBytes, keyId, teamId, topic, production);
+                } catch (IOException e) {
+                    LOG.warn("Error instantiating APNS options from resource {}", privateKey, e);
+                }
+            }
+
+            // Use PKCS#12 keystore as fall-back
+            String keystore = optProperty(properties, DriveEventsAPNProperty.keystore, optionals);
             if (Strings.isNotEmpty(keystore)) {
                 String password = getProperty(properties, DriveEventsAPNProperty.password, optionals);
                 boolean production = Boolean.parseBoolean(getProperty(properties, DriveEventsAPNProperty.production, optionals));
@@ -191,7 +193,7 @@ public class APNDriveEventsActivator extends HousekeepingActivator {
                 }
             }
         } catch (OXException e) {
-            // nothing to do
+            LOG.debug("Error while creating APN access", e);
         }
         return null;
     }
@@ -215,6 +217,22 @@ public class APNDriveEventsActivator extends HousekeepingActivator {
         return result;
     }
 
+    /**
+     * Get the given property from the {@link Properties} object or <code>null</code>
+     *
+     * @param properties The {@link Properties} object
+     * @param prop The {@link Property} to return
+     * @param optional The optional
+     * @return The string value of the property or <code>null</code> in case property is missing
+     */
+    private String optProperty(Properties properties, Property prop, Map<String, String> optional) {
+        String result = properties.getProperty(prop.getFQPropertyName(optional));
+        if (null == result) {
+            LOG.debug("Missing required property from fragment: {}", prop.getFQPropertyName());
+        }
+        return result;
+    }
+
     @Override
     protected void stopBundle() throws Exception {
         LOG.info("stopping bundle: {}", context.getBundle().getSymbolicName());
