@@ -1,49 +1,21 @@
 /*
+ * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @license AGPL-3.0
  *
- *    OPEN-XCHANGE legal information
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *    All intellectual property rights in the Software are protected by
- *    international copyright laws.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Affero General Public License for more details.
  *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with OX App Suite.  If not, see <https://www.gnu.org/licenses/agpl-3.0.txt>.
  *
- *    In some countries OX, OX Open-Xchange, open xchange and OXtender
- *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the OX Software GmbH group of companies.
- *    The use of the Logos is not covered by the GNU General Public License.
- *    Instead, you are allowed to use these Logos according to the terms and
- *    conditions of the Creative Commons License, Version 2.5, Attribution,
- *    Non-commercial, ShareAlike, and the interpretation of the term
- *    Non-commercial applicable to the aforementioned license is published
- *    on the web site http://www.open-xchange.com/EN/legal/index.html.
- *
- *    Please make sure that third-party modules and libraries are used
- *    according to their respective licenses.
- *
- *    Any modifications to this package must retain all copyright notices
- *    of the original copyright holder(s) for the original code used.
- *
- *    After any such modifications, the original and derivative code shall remain
- *    under the copyright of the copyright holder(s) and/or original author(s)per
- *    the Attribution and Assignment Agreement that can be located at
- *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
- *    given Attribution for the derivative code and a license granting use.
- *
- *     Copyright (C) 2016-2020 OX Software GmbH
- *     Mail: info@open-xchange.com
- *
- *
- *     This program is free software; you can redistribute it and/or modify it
- *     under the terms of the GNU General Public License, Version 2 as published
- *     by the Free Software Foundation.
- *
- *     This program is distributed in the hope that it will be useful, but
- *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- *     for more details.
- *
- *     You should have received a copy of the GNU General Public License along
- *     with this program; if not, write to the Free Software Foundation, Inc., 59
- *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Any use of the work other than as authorized under this license or copyright law is prohibited.
  *
  */
 
@@ -220,6 +192,9 @@ public class DriveServiceImpl implements DriveService {
                 }
                 throw e;
             }
+            if (driveSession.getStorage().isInTrash(rootFolder, DriveConstants.ROOT_PATH)) {
+                return getErrorResult(driveSession, DriveExceptionCodes.NOT_SYNCHRONIZABLE_DIRECTORY_IN_TRASH.create(session.getRootFolderID()), true);
+            }
             if (driveSession.getStorage().isExcludedSubfolder(rootFolder, DriveConstants.ROOT_PATH)) {
                 return getErrorResult(driveSession, DriveExceptionCodes.NOT_SYNCHRONIZABLE_DIRECTORY.create(session.getRootFolderID()), true);
             }
@@ -329,6 +304,9 @@ public class DriveServiceImpl implements DriveService {
                 }
                 throw e;
             }
+            if (driveSession.getStorage().isInTrash(rootFolder, DriveConstants.ROOT_PATH)) {
+                return getErrorResult(driveSession, path, DriveExceptionCodes.NOT_SYNCHRONIZABLE_DIRECTORY_IN_TRASH.create(session.getRootFolderID()), true);
+            }
             if (driveSession.getStorage().isExcludedSubfolder(rootFolder, DriveConstants.ROOT_PATH)) {
                 return getErrorResult(driveSession, path, DriveExceptionCodes.NOT_SYNCHRONIZABLE_DIRECTORY.create(session.getRootFolderID()), true);
             }
@@ -631,13 +609,16 @@ public class DriveServiceImpl implements DriveService {
          */
         if (DriveClientType.ANDROID.equals(session.getDriveSession().getClientType()) || DriveClientType.IOS.equals(session.getDriveSession().getClientType())) {
             int zeroByteClientVersionCount = 0;
+            boolean updateForNonZeroByteFile = false;
             for (Entry<String, ThreeWayComparison<FileVersion>> entry : mapper) {
                 ThreeWayComparison<FileVersion> comparison = entry.getValue();
                 if (null != comparison.getClientVersion() && DriveConstants.EMPTY_MD5.equals(comparison.getClientVersion().getChecksum()) && null != comparison.getOriginalVersion()) {
                     zeroByteClientVersionCount++;
+                    updateForNonZeroByteFile = updateForNonZeroByteFile ||
+                        null != comparison.getServerVersion() && false == DriveConstants.EMPTY_MD5.equals(comparison.getServerVersion().getChecksum());
                 }
             }
-            if (0 < zeroByteClientVersionCount && zeroByteClientVersionCount == clientVersions.size()) {
+            if (0 < zeroByteClientVersionCount && zeroByteClientVersionCount == clientVersions.size() && updateForNonZeroByteFile) {
                 OXException error = DriveExceptionCodes.ZERO_BYTE_FILES.create(path);
                 LOG.warn("Client synchronization aborted for {}", session, error);
                 IntermediateSyncResult<FileVersion> errorResult = new IntermediateSyncResult<FileVersion>();
