@@ -1,49 +1,21 @@
 /*
+ * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @license AGPL-3.0
  *
- *    OPEN-XCHANGE legal information
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *    All intellectual property rights in the Software are protected by
- *    international copyright laws.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Affero General Public License for more details.
  *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with OX App Suite.  If not, see <https://www.gnu.org/licenses/agpl-3.0.txt>.
  *
- *    In some countries OX, OX Open-Xchange, open xchange and OXtender
- *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the OX Software GmbH group of companies.
- *    The use of the Logos is not covered by the GNU General Public License.
- *    Instead, you are allowed to use these Logos according to the terms and
- *    conditions of the Creative Commons License, Version 2.5, Attribution,
- *    Non-commercial, ShareAlike, and the interpretation of the term
- *    Non-commercial applicable to the aforementioned license is published
- *    on the web site http://www.open-xchange.com/EN/legal/index.html.
- *
- *    Please make sure that third-party modules and libraries are used
- *    according to their respective licenses.
- *
- *    Any modifications to this package must retain all copyright notices
- *    of the original copyright holder(s) for the original code used.
- *
- *    After any such modifications, the original and derivative code shall remain
- *    under the copyright of the copyright holder(s) and/or original author(s)per
- *    the Attribution and Assignment Agreement that can be located at
- *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
- *    given Attribution for the derivative code and a license granting use.
- *
- *     Copyright (C) 2016-2020 OX Software GmbH
- *     Mail: info@open-xchange.com
- *
- *
- *     This program is free software; you can redistribute it and/or modify it
- *     under the terms of the GNU General Public License, Version 2 as published
- *     by the Free Software Foundation.
- *
- *     This program is distributed in the hope that it will be useful, but
- *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- *     for more details.
- *
- *     You should have received a copy of the GNU General Public License along
- *     with this program; if not, write to the Free Software Foundation, Inc., 59
- *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Any use of the work other than as authorized under this license or copyright law is prohibited.
  *
  */
 
@@ -53,6 +25,7 @@ import static com.openexchange.drive.impl.DriveConstants.ROOT_PATH;
 import java.sql.Connection;
 import java.text.Collator;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Collections;
 import java.util.Comparator;
 import java.util.List;
@@ -104,13 +77,17 @@ import com.openexchange.folderstorage.FolderServiceDecorator;
 import com.openexchange.folderstorage.UserizedFolder;
 import com.openexchange.folderstorage.database.contentType.InfostoreContentType;
 import com.openexchange.groupware.container.FolderObject;
+import com.openexchange.groupware.contexts.Context;
 import com.openexchange.java.Collators;
-import com.openexchange.java.Reference;
+import com.openexchange.java.util.Pair;
 import com.openexchange.session.Session;
 import com.openexchange.share.LinkUpdate;
 import com.openexchange.share.ShareTarget;
+import com.openexchange.share.core.tools.ShareTool;
 import com.openexchange.share.notification.Entities;
+import com.openexchange.share.notification.Entities.Permission;
 import com.openexchange.tools.iterator.SearchIterator;
+import com.openexchange.user.UserService;
 
 /**
  * {@link DriveUtilityImpl}
@@ -246,11 +223,7 @@ public class DriveUtilityImpl implements DriveUtility {
 
             @Override
             public JSONObject call() throws OXException {
-                try {
-                    return getFileMetadata(syncSession, path, fileVersion);
-                } catch (JSONException e) {
-                    throw DriveExceptionCodes.IO_ERROR.create(e, e.getMessage());
-                }
+                return getFileMetadata(syncSession, path, fileVersion);
             }
         });
     }
@@ -262,30 +235,25 @@ public class DriveUtilityImpl implements DriveUtility {
 
             @Override
             public JSONObject call() throws OXException {
-                try {
-                    return getDirectoryMetadata(syncSession, directoryVersion);
-                } catch (JSONException e) {
-                    throw DriveExceptionCodes.IO_ERROR.create(e, e.getMessage());
-                }
+                return getDirectoryMetadata(syncSession, directoryVersion);
             }
         });
     }
 
     @Override
-    public void updateFile(DriveSession session, final String path, final FileVersion fileVersion, JSONObject jsonObject, NotificationParameters parameters) throws OXException {
+    public JSONObject updateFile(DriveSession session, final String path, final FileVersion fileVersion, JSONObject jsonObject, NotificationParameters parameters) throws OXException {
         List<Field> fields = new ArrayList<>();
         final File metadata = FileMetadataParser.parse(jsonObject, fields);
         if (fields.isEmpty()) {
-            return;
+            return getFileMetadata(session, path, fileVersion);
         }
         final SyncSession syncSession = new SyncSession(session);
         syncSession.trace("About to update metadata for file [" + fileVersion + "]: " + jsonObject);
         final boolean notify = null != parameters.getNotificationTransport();
-        final Reference<ShareTarget> targetReference = new Reference<ShareTarget>();
-        Entities entities = syncSession.getStorage().wrapInTransaction(new StorageOperation<Entities>() {
+        Pair<File, Entities> result = syncSession.getStorage().wrapInTransaction(new StorageOperation<Pair<File, Entities>>() {
 
             @Override
-            public Entities call() throws OXException {
+            public Pair<File, Entities> call() throws OXException {
                 /*
                  * get the original file
                  */
@@ -302,40 +270,38 @@ public class DriveUtilityImpl implements DriveUtility {
                 List<Field> updatedFields = Collections.singletonList(Field.OBJECT_PERMISSIONS);
                 DefaultFile updatedFile = new DefaultFile(file);
                 updatedFile.setObjectPermissions(metadata.getObjectPermissions());
-                String fileID = syncSession.getStorage().getFileAccess().saveFileMetadata(updatedFile, file.getSequenceNumber(), updatedFields);
+                String fileID = syncSession.getStorage().getFileAccess().saveFileMetadata(updatedFile, file.getSequenceNumber(), updatedFields, true, false);
                 /*
-                 * re-get updated file to determine added permissions as needed
+                 * re-get updated file & determine added permissions as needed
                  */
-                if (notify) {
-                    File reloadedFile = syncSession.getStorage().getFileAccess().getFileMetadata(fileID, FileStorageFileAccess.CURRENT_VERSION);
-                    targetReference.setValue(new ShareTarget(DriveConstants.FILES_MODULE, reloadedFile.getFolderId(), reloadedFile.getId()));
-                    return ShareHelper.getAddedPermissions(file, reloadedFile);
-                }
-                return null;
+                File reloadedFile = syncSession.getStorage().getFileAccess().getFileMetadata(fileID, FileStorageFileAccess.CURRENT_VERSION);
+                Entities addedPermissions = notify ? ShareHelper.getAddedPermissions(file, reloadedFile) : null;
+                return new Pair<File, Entities>(reloadedFile, addedPermissions);
             }
         });
         syncSession.trace("Metadata for file [" + fileVersion + "] updated successfully.");
         /*
-         * send notifications if needed
+         * send notifications if needed & return updated metadata
          */
-        if (notify && null != entities && 0 < entities.size()) {
-            ShareHelper shareHelper = new ShareHelper(syncSession);
-            parameters.addWarnings(shareHelper.sendNotifications(
-                targetReference.getValue(), parameters.getNotificationTransport(), parameters.getNotificationMessage(), entities));
+        Entities entities = notify ? filterAnonymous(syncSession.getServerSession().getContext(), result.getSecond()) : null;
+        if (null != entities && 0 < entities.size()) {
+            ShareTarget shareTarget = new ShareTarget(DriveConstants.FILES_MODULE, result.getFirst().getFolderId(), result.getFirst().getId());
+            parameters.addWarnings(new ShareHelper(syncSession).sendNotifications(
+                shareTarget, parameters.getNotificationTransport(), parameters.getNotificationMessage(), entities));
         }
+        return getFileMetadata(syncSession, result.getFirst());        
     }
-
+    
     @Override
-    public void updateDirectory(DriveSession session, final DirectoryVersion directoryVersion, JSONObject jsonObject, final boolean cascadePermissions, NotificationParameters parameters) throws OXException {
+    public JSONObject updateDirectory(DriveSession session, final DirectoryVersion directoryVersion, JSONObject jsonObject, final boolean cascadePermissions, NotificationParameters parameters) throws OXException {
         final FileStorageFolder folder = DirectoryMetadataParser.parse(jsonObject);
         final SyncSession syncSession = new SyncSession(session);
         syncSession.trace("About to update metadata for directory [" + directoryVersion + "]: " + jsonObject);
         final boolean notify = null != parameters.getNotificationTransport();
-        final Reference<ShareTarget> targetReference = new Reference<>();
-        Entities entities = syncSession.getStorage().wrapInTransaction(new StorageOperation<Entities>() {
+        Pair<FileStorageFolder, Entities> result = syncSession.getStorage().wrapInTransaction(new StorageOperation<Pair<FileStorageFolder, Entities>>() {
 
             @Override
-            public Entities call() throws OXException {
+            public Pair<FileStorageFolder, Entities> call() throws OXException {
                 /*
                  * get the original directory version
                  */
@@ -349,25 +315,28 @@ public class DriveUtilityImpl implements DriveUtility {
                 updatedFolder.setPermissions(folder.getPermissions());
                 folderID = syncSession.getStorage().getFolderAccess().updateFolder(folderID, updatedFolder, cascadePermissions);
                 /*
-                 * re-get updated folder to determine added permissions as needed
+                 * re-get updated folder & determine added permissions as needed
                  */
-                if (notify) {
-                    FileStorageFolder reloadedFolder = syncSession.getStorage().getFolderAccess().getFolder(folderID);
-                    targetReference.setValue(new ShareTarget(DriveConstants.FILES_MODULE, reloadedFolder.getId()));
-                    return ShareHelper.getAddedPermissions(originalFolder, reloadedFolder);
-                }
-                return null;
+                FileStorageFolder reloadedFolder = syncSession.getStorage().getFolderAccess().getFolder(folderID);
+                Entities addedPermissions = notify ? ShareHelper.getAddedPermissions(originalFolder, reloadedFolder) : null;
+                return new Pair<FileStorageFolder, Entities>(reloadedFolder, addedPermissions);
             }
         });
         syncSession.trace("Metadata for directory [" + directoryVersion + "] updated successfully.");
         /*
-         * send notifications if needed
+         * send notifications if needed & return updated metadata
          */
-        if (notify && null != entities && 0 < entities.size()) {
-            ShareHelper shareHelper = new ShareHelper(syncSession);
-            parameters.addWarnings(shareHelper.sendNotifications(
-                targetReference.getValue(), parameters.getNotificationTransport(), parameters.getNotificationMessage(), entities));
+        Entities entities = notify ? filterAnonymous(syncSession.getServerSession().getContext(), result.getSecond()) : null;
+        if (null != entities && 0 < entities.size()) {
+            ShareTarget shareTarget = new ShareTarget(DriveConstants.FILES_MODULE, result.getFirst().getId());
+            parameters.addWarnings(new ShareHelper(syncSession).sendNotifications(
+                shareTarget, parameters.getNotificationTransport(), parameters.getNotificationMessage(), entities));
         }
+        List<DirectoryChecksum> checksums = ChecksumProvider.getChecksums(syncSession, Arrays.asList(new String[] { result.getFirst().getId() }));
+        if (null == checksums || 0 == checksums.size()) {
+            throw DriveExceptionCodes.PATH_NOT_FOUND.create(directoryVersion.getPath());
+        }
+        return getDirectoryMetadata(syncSession, result.getFirst(), directoryVersion.getPath(), checksums.get(0).getChecksum());
     }
 
     @Override
@@ -518,33 +487,49 @@ public class DriveUtilityImpl implements DriveUtility {
         return jsonArray;
     }
 
-    JSONObject getFileMetadata(SyncSession session, String path, FileVersion fileVersion) throws OXException, JSONException {
+    JSONObject getFileMetadata(SyncSession session, String path, FileVersion fileVersion) throws OXException {
         File file = session.getStorage().getFileByName(path, fileVersion.getName(), true);
         if (null == file || false == ChecksumProvider.matches(session, file, fileVersion.getChecksum())) {
             throw DriveExceptionCodes.FILEVERSION_NOT_FOUND.create(fileVersion.getName(), fileVersion.getChecksum(), path);
         }
-        JSONObject jsonObject = new JsonFileMetadata(session, session.getStorage().getFile(file.getId())).build();
-        jsonObject.put(PATH, session.getStorage().getPath(file.getFolderId()));
-        jsonObject.put(CHECKSUM, ChecksumProvider.getChecksum(session, file).getChecksum());
-        return jsonObject;
+        return getFileMetadata(session, file);
     }
+    
+    private static JSONObject getFileMetadata(SyncSession session, File file) throws OXException {
+        try {
+            JSONObject jsonObject = new JsonFileMetadata(session, session.getStorage().getFile(file.getId())).build();
+            jsonObject.put(PATH, session.getStorage().getPath(file.getFolderId()));
+            jsonObject.put(CHECKSUM, ChecksumProvider.getChecksum(session, file).getChecksum());
+            return jsonObject;
+        } catch (JSONException e) {
+            throw DriveExceptionCodes.IO_ERROR.create(e, e.getMessage());
+        }
+    }    
 
-    JSONObject getDirectoryMetadata(SyncSession session, DirectoryVersion directoryVersion) throws OXException, JSONException {
+    JSONObject getDirectoryMetadata(SyncSession session, DirectoryVersion directoryVersion) throws OXException {
         ServerDirectoryVersion serverVersion = ServerDirectoryVersion.valueOf(directoryVersion, session);
         FileStorageFolder folder = session.getStorage().getFolder(serverVersion.getPath());
-        JSONObject jsonObject = new JsonDirectoryMetadata(session, folder).build(false);
-        jsonObject.put(CHECKSUM, serverVersion.getChecksum());
-        jsonObject.put(PATH, serverVersion.getPath());
-        jsonObject.put(NAME, folder.getName());
-        if (null != folder.getCreationDate()) {
-            jsonObject.put(CREATED, folder.getCreationDate().getTime());
+        return getDirectoryMetadata(session, folder, serverVersion.getPath(), serverVersion.getChecksum());
+    }
+
+    private static JSONObject getDirectoryMetadata(SyncSession session, FileStorageFolder folder, String path, String checksum) throws OXException {
+        try {
+            JSONObject jsonObject = new JsonDirectoryMetadata(session, folder).build(false);
+            jsonObject.put(CHECKSUM, checksum);
+            jsonObject.put(PATH, path);
+            jsonObject.put(NAME, folder.getName());
+            if (null != folder.getCreationDate()) {
+                jsonObject.put(CREATED, folder.getCreationDate().getTime());
+            }
+            if (null != folder.getLastModifiedDate()) {
+                jsonObject.put(MODIFIED, folder.getLastModifiedDate().getTime());
+            }
+            jsonObject.put(CREATED_BY, folder.getCreatedBy());
+            jsonObject.put(MODIFIED_BY, folder.getModifiedBy());
+            return jsonObject;
+        } catch (JSONException e) {
+            throw DriveExceptionCodes.IO_ERROR.create(e, e.getMessage());
         }
-        if (null != folder.getLastModifiedDate()) {
-            jsonObject.put(MODIFIED, folder.getLastModifiedDate().getTime());
-        }
-        jsonObject.put(CREATED_BY, folder.getCreatedBy());
-        jsonObject.put(MODIFIED_BY, folder.getModifiedBy());
-        return jsonObject;
     }
 
     @Override
@@ -806,4 +791,29 @@ public class DriveUtilityImpl implements DriveUtility {
         }
     }
 
+    private static Entities filterAnonymous(Context context, Entities entities) {
+        if (null == entities) {
+            return null;
+        }
+        try {
+            Entities filteredEntities = new Entities();
+            for (Integer id : entities.getGroups()) {
+                Permission permission = filteredEntities.getGroupPermissionBits(id.intValue());
+                filteredEntities.addUser(id.intValue(), permission.getType(), permission.getPermissions());
+            }
+            UserService userService = DriveServiceLookup.getService(UserService.class, true);
+            for (Integer id : entities.getUsers()) {
+                if (ShareTool.isAnonymousGuest(userService.getUser(id.intValue(), context))) {
+                    continue;
+                }
+                Permission permission = filteredEntities.getUserPermissionBits(id.intValue());
+                filteredEntities.addUser(id.intValue(), permission.getType(), permission.getPermissions());
+            }
+            return filteredEntities;
+        } catch (Exception e) {
+            LOG.warn("Unable to filter anonymous entities, using entities list as-is", e);
+            return entities;
+        }
+    }
+
 }
