@@ -1,49 +1,21 @@
 /*
+ * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @license AGPL-3.0
  *
- *    OPEN-XCHANGE legal information
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *    All intellectual property rights in the Software are protected by
- *    international copyright laws.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Affero General Public License for more details.
  *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with OX App Suite.  If not, see <https://www.gnu.org/licenses/agpl-3.0.txt>.
  *
- *    In some countries OX, OX Open-Xchange, open xchange and OXtender
- *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the OX Software GmbH group of companies.
- *    The use of the Logos is not covered by the GNU General Public License.
- *    Instead, you are allowed to use these Logos according to the terms and
- *    conditions of the Creative Commons License, Version 2.5, Attribution,
- *    Non-commercial, ShareAlike, and the interpretation of the term
- *    Non-commercial applicable to the aforementioned license is published
- *    on the web site http://www.open-xchange.com/EN/legal/index.html.
- *
- *    Please make sure that third-party modules and libraries are used
- *    according to their respective licenses.
- *
- *    Any modifications to this package must retain all copyright notices
- *    of the original copyright holder(s) for the original code used.
- *
- *    After any such modifications, the original and derivative code shall remain
- *    under the copyright of the copyright holder(s) and/or original author(s)per
- *    the Attribution and Assignment Agreement that can be located at
- *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
- *    given Attribution for the derivative code and a license granting use.
- *
- *     Copyright (C) 2016-2020 OX Software GmbH
- *     Mail: info@open-xchange.com
- *
- *
- *     This program is free software; you can redistribute it and/or modify it
- *     under the terms of the GNU General Public License, Version 2 as published
- *     by the Free Software Foundation.
- *
- *     This program is distributed in the hope that it will be useful, but
- *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- *     for more details.
- *
- *     You should have received a copy of the GNU General Public License along
- *     with this program; if not, write to the Free Software Foundation, Inc., 59
- *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Any use of the work other than as authorized under this license or copyright law is prohibited.
  *
  */
 
@@ -116,23 +88,19 @@ public class UploadHelper {
         super();
         this.session = session;
     }
-    
+
     public File perform(final String path, final FileVersion originalVersion, final FileVersion newVersion, final InputStream uploadStream,
         final String contentType, final long offset, final long totalLength, final Date created, final Date modified) throws OXException {
         /*
          * Try to save directly if applicable or required
          */
         if (false == useSeparateUpload(path, originalVersion, newVersion, offset, totalLength)) {
-            Entry<File, String> uploadEntry = session.getStorage().wrapInTransaction(new StorageOperation<Entry<File, String>>() {
-
-                @Override
-                public Entry<File, String> call() throws OXException {
-                    if (null != originalVersion) {
-                        File originalFile = ServerFileVersion.valueOf(originalVersion, path, session).getFile();
-                        return saveOptimistically(path, newVersion, uploadStream, contentType, created, modified, originalFile.getId(), originalFile.getSequenceNumber());
-                    }
-                    return saveOptimistically(path, newVersion, uploadStream, contentType, created, modified);
+            Entry<File, String> uploadEntry = session.getStorage().wrapInTransaction(() -> {
+                if (null != originalVersion) {
+                    File originalFile = ServerFileVersion.valueOf(originalVersion, path, session).getFile();
+                    return saveOptimistically(path, newVersion, uploadStream, contentType, created, modified, originalFile.getId(), originalFile.getSequenceNumber());
                 }
+                return saveOptimistically(path, newVersion, uploadStream, contentType, created, modified);
             });
             /*
              * validate checksum
@@ -265,7 +233,7 @@ public class UploadHelper {
                     file.setFileName(alternativeName);
                     file.setTitle(alternativeName);
                     fields.add(Field.TITLE);
-                    session.getStorage().getFileAccess().saveFileMetadata(file, sequenceNumber, fields);
+                    session.getStorage().getFileAccess().saveFileMetadata(file, sequenceNumber, fields, true, false);
                     return file;
                 }
                 /*
@@ -276,7 +244,7 @@ public class UploadHelper {
                 fields.add(Field.FOLDER_ID);
                 file.setTitle(newVersion.getName());
                 fields.add(Field.TITLE);
-                session.getStorage().getFileAccess().saveFileMetadata(file, sequenceNumber, fields);
+                session.getStorage().getFileAccess().saveFileMetadata(file, sequenceNumber, fields, true, false);
                 return file;
             }
         });
@@ -506,7 +474,7 @@ public class UploadHelper {
             meta.put(KEY_CHECKSUM_STATE, checksumStateBinary);
         }
         updateFile.setMeta(meta);
-        fileAccess.saveFileMetadata(updateFile, file.getSequenceNumber(), modifiedColumns);
+        fileAccess.saveFileMetadata(updateFile, file.getSequenceNumber(), modifiedColumns, true, false);
         return true;
 
     }
@@ -571,7 +539,7 @@ public class UploadHelper {
             session.trace("Using existing upload file at " + DriveUtils.combine(uploadPath, uploadFileName) +
                 " [" + uploadFile.getId() + "], current size: " + uploadFile.getFileSize() +
                 ", last modified: " + (null != uploadFile.getLastModified() ?
-                    DriveConstants.LOG_DATE_FORMAT.get().format(uploadFile.getLastModified()) : "(unknown)") + 
+                    DriveConstants.LOG_DATE_FORMAT.get().format(uploadFile.getLastModified()) : "(unknown)") +
                 ", checksum state: " + (null != checksumState ? Arrays.toString(checksumState) : "(unknown)"));
 
         }
@@ -654,21 +622,16 @@ public class UploadHelper {
         final List<Field> fields = Arrays.asList(Field.FILENAME, Field.FILE_SIZE);
         SearchIterator<File> searchIterator = null;
         try {
-            searchIterator = session.getStorage().wrapInTransaction(new StorageOperation<SearchIterator<File>>() {
-
-                @Override
-                public SearchIterator<File> call() throws OXException {
-                    if (session.getStorage().supports(new FolderID(folderID), FileStorageCapability.SEARCH_BY_TERM)) {
-                        return session.getStorage().getFileAccess().search(Collections.singletonList(folderID),
-                            getSearchTermForUploadFiles(fileVersions), fields, null,
-                            SortDirection.DEFAULT, FileStorageFileAccess.NOT_SET, FileStorageFileAccess.NOT_SET);
-                    } else {
-                        String pattern = 1 == fileVersions.size() ?
-                            getUploadFilename(fileVersions.get(0).getChecksum()) : "*" + DriveConstants.FILEPART_EXTENSION;
-                        return session.getStorage().getFileAccess().search(pattern, fields, folderID, null,
-                            SortDirection.DEFAULT, FileStorageFileAccess.NOT_SET, FileStorageFileAccess.NOT_SET);
-                    }
+            searchIterator = session.getStorage().wrapInTransaction(() -> {
+                if (session.getStorage().supports(new FolderID(folderID), FileStorageCapability.SEARCH_BY_TERM)) {
+                    return session.getStorage().getFileAccess().search(Collections.singletonList(folderID),
+                        getSearchTermForUploadFiles(fileVersions), fields, null,
+                        SortDirection.DEFAULT, FileStorageFileAccess.NOT_SET, FileStorageFileAccess.NOT_SET);
                 }
+                String pattern = 1 == fileVersions.size() ?
+                    getUploadFilename(fileVersions.get(0).getChecksum()) : "*" + DriveConstants.FILEPART_EXTENSION;
+                return session.getStorage().getFileAccess().search(pattern, fields, folderID, null,
+                    SortDirection.DEFAULT, FileStorageFileAccess.NOT_SET, FileStorageFileAccess.NOT_SET);
             });
             while (searchIterator.hasNext()) {
                 File file = searchIterator.next();
@@ -689,8 +652,8 @@ public class UploadHelper {
     }
 
     /**
-     * Gets a value indicating whether the upload should be stored in a separate temporary upload file or not, depending on the target 
-     * folder and indicated file. 
+     * Gets a value indicating whether the upload should be stored in a separate temporary upload file or not, depending on the target
+     * folder and indicated file.
      *
      * @param path The path of the destination directory
      * @param originalVersion The original file version being replaced, or <code>null</code> for new file versions
@@ -707,11 +670,11 @@ public class UploadHelper {
             return true;
         }
         /*
-         * do save directly, if no permissions in folder and no alternative upload location available          
+         * do save directly, if no permissions in folder and no alternative upload location available
          */
         FileStoragePermission permission = session.getStorage().getFolder(path, true).getOwnPermission();
         if ((FileStoragePermission.CREATE_OBJECTS_IN_FOLDER > permission.getFolderPermission() ||
-            FileStoragePermission.WRITE_OWN_OBJECTS > permission.getWritePermission()) && 
+            FileStoragePermission.WRITE_OWN_OBJECTS > permission.getWritePermission()) &&
             false == session.getTemp().supported()) {
             return false;
         }
