@@ -1,49 +1,21 @@
 /*
+ * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @license AGPL-3.0
  *
- *    OPEN-XCHANGE legal information
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *    All intellectual property rights in the Software are protected by
- *    international copyright laws.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Affero General Public License for more details.
  *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with OX App Suite.  If not, see <https://www.gnu.org/licenses/agpl-3.0.txt>.
  *
- *    In some countries OX, OX Open-Xchange, open xchange and OXtender
- *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the OX Software GmbH group of companies.
- *    The use of the Logos is not covered by the GNU General Public License.
- *    Instead, you are allowed to use these Logos according to the terms and
- *    conditions of the Creative Commons License, Version 2.5, Attribution,
- *    Non-commercial, ShareAlike, and the interpretation of the term
- *    Non-commercial applicable to the aforementioned license is published
- *    on the web site http://www.open-xchange.com/EN/legal/index.html.
- *
- *    Please make sure that third-party modules and libraries are used
- *    according to their respective licenses.
- *
- *    Any modifications to this package must retain all copyright notices
- *    of the original copyright holder(s) for the original code used.
- *
- *    After any such modifications, the original and derivative code shall remain
- *    under the copyright of the copyright holder(s) and/or original author(s)per
- *    the Attribution and Assignment Agreement that can be located at
- *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
- *    given Attribution for the derivative code and a license granting use.
- *
- *     Copyright (C) 2016-2020 OX Software GmbH
- *     Mail: info@open-xchange.com
- *
- *
- *     This program is free software; you can redistribute it and/or modify it
- *     under the terms of the GNU General Public License, Version 2 as published
- *     by the Free Software Foundation.
- *
- *     This program is distributed in the hope that it will be useful, but
- *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- *     for more details.
- *
- *     You should have received a copy of the GNU General Public License along
- *     with this program; if not, write to the Free Software Foundation, Inc., 59
- *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Any use of the work other than as authorized under this license or copyright law is prohibited.
  *
  */
 
@@ -63,10 +35,12 @@ import com.openexchange.drive.DriveExceptionCodes;
 import com.openexchange.drive.impl.DriveUtils;
 import com.openexchange.drive.impl.internal.SyncSession;
 import com.openexchange.exception.OXException;
+import com.openexchange.file.storage.DefaultFileStorageFolder;
 import com.openexchange.file.storage.File;
 import com.openexchange.file.storage.File.Field;
 import com.openexchange.file.storage.FileStorageCapability;
 import com.openexchange.file.storage.FileStorageFolder;
+import com.openexchange.file.storage.FileStorageFolderPermissionType;
 import com.openexchange.file.storage.FileStoragePermission;
 import com.openexchange.file.storage.TypeAware;
 import com.openexchange.file.storage.composition.FolderID;
@@ -231,7 +205,14 @@ public class JsonDirectoryMetadata extends AbstractJsonMetadata {
         }
         JSONArray jsonArray = new JSONArray(permissions.size());
         for (FileStoragePermission permission : permissions) {
-            jsonArray.put(extended ? getExtendedJSONPermission(permission) : getJSONPermission(permission));
+            if (extended) {
+                jsonArray.put(getExtendedJSONPermission(permission));
+            } else {
+                if (FileStorageFolderPermissionType.INHERITED.equals(permission.getType())) {
+                    continue; // skip inherited permissions for plain permission array implicitly
+                }
+                jsonArray.put(getJSONPermission(permission));
+            }
         }
         return jsonArray;
     }
@@ -249,6 +230,10 @@ public class JsonDirectoryMetadata extends AbstractJsonMetadata {
         jsonObject.put("entity", permission.getEntity());
         jsonObject.put("bits", Permissions.createPermissionBits(permission.getFolderPermission(), permission.getReadPermission(),
             permission.getWritePermission(), permission.getDeletePermission(), permission.isAdmin()));
+        if (FileStorageFolderPermissionType.INHERITED.equals(permission.getType())) {
+            jsonObject.put("isInherited", true);
+            jsonObject.put("isInheritedFrom", permission.getPermissionLegator());
+        }
         if (permission.isGroup()) {
             jsonObject.put("type", "group");
             addGroupInfo(jsonObject, session.getPermissionResolver().getGroup(permission.getEntity()));
@@ -266,7 +251,13 @@ public class JsonDirectoryMetadata extends AbstractJsonMetadata {
 
                 jsonObject.put("type", guest.getRecipientType().toString().toLowerCase());
                 if (RecipientType.ANONYMOUS.equals(guest.getRecipientType())) {
-                    addShareInfo(jsonObject, session.getPermissionResolver().getShare(folder, permission.getEntity()));
+                    if (FileStorageFolderPermissionType.INHERITED.equals(permission.getType())) {
+                        DefaultFileStorageFolder legator = new DefaultFileStorageFolder();
+                        legator.setId(permission.getPermissionLegator());
+                        addShareInfo(jsonObject, session.getPermissionResolver().getShare(legator, permission.getEntity()));
+                    } else {
+                        addShareInfo(jsonObject, session.getPermissionResolver().getShare(folder, permission.getEntity()));
+                    }
                 } else {
                     addUserInfo(jsonObject, user);
                 }
@@ -283,6 +274,9 @@ public class JsonDirectoryMetadata extends AbstractJsonMetadata {
         if (null != permissions && 0 < permissions.size()) {
             int userID = session.getServerSession().getUserId();
             for (FileStoragePermission permission : permissions) {
+                if (FileStorageFolderPermissionType.INHERITED.equals(permission.getType())) {
+                    continue; // skip inherited permissions for plain permission array implicitly
+                }
                 if (permission.getEntity() != userID) {
                     return true;
                 }
