@@ -1,49 +1,21 @@
 /*
+ * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @license AGPL-3.0
  *
- *    OPEN-XCHANGE legal information
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *    All intellectual property rights in the Software are protected by
- *    international copyright laws.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Affero General Public License for more details.
  *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with OX App Suite.  If not, see <https://www.gnu.org/licenses/agpl-3.0.txt>.
  *
- *    In some countries OX, OX Open-Xchange, open xchange and OXtender
- *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the OX Software GmbH group of companies.
- *    The use of the Logos is not covered by the GNU General Public License.
- *    Instead, you are allowed to use these Logos according to the terms and
- *    conditions of the Creative Commons License, Version 2.5, Attribution,
- *    Non-commercial, ShareAlike, and the interpretation of the term
- *    Non-commercial applicable to the aforementioned license is published
- *    on the web site http://www.open-xchange.com/EN/legal/index.html.
- *
- *    Please make sure that third-party modules and libraries are used
- *    according to their respective licenses.
- *
- *    Any modifications to this package must retain all copyright notices
- *    of the original copyright holder(s) for the original code used.
- *
- *    After any such modifications, the original and derivative code shall remain
- *    under the copyright of the copyright holder(s) and/or original author(s)per
- *    the Attribution and Assignment Agreement that can be located at
- *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
- *    given Attribution for the derivative code and a license granting use.
- *
- *     Copyright (C) 2016-2020 OX Software GmbH
- *     Mail: info@open-xchange.com
- *
- *
- *     This program is free software; you can redistribute it and/or modify it
- *     under the terms of the GNU General Public License, Version 2 as published
- *     by the Free Software Foundation.
- *
- *     This program is distributed in the hope that it will be useful, but
- *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- *     for more details.
- *
- *     You should have received a copy of the GNU General Public License along
- *     with this program; if not, write to the Free Software Foundation, Inc., 59
- *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Any use of the work other than as authorized under this license or copyright law is prohibited.
  *
  */
 
@@ -403,7 +375,7 @@ public class DriveStorage {
                 session.trace(this.toString() + "mv " + combine(getPath(file.getFolderId()), file.getFileName()) + " " +
                     combine(getPath(movedFile.getFolderId()), movedFile.getFileName()));
             }
-            getFileAccess().saveFileMetadata(movedFile, file.getSequenceNumber(), updatedFields);
+            getFileAccess().saveFileMetadata(movedFile, file.getSequenceNumber(), updatedFields, true, false);
         }
         return movedFile;
     }
@@ -483,8 +455,9 @@ public class DriveStorage {
             /*
              * perform move / rename
              */
-            folderID = oldName.equals(newName) ? getFolderAccess().moveFolder(folderID, newParentFolder.getId()) :
-                getFolderAccess().moveFolder(folderID, newParentFolder.getId(), newName);
+            boolean ignoreWarnings = true;
+            folderID = oldName.equals(newName) ? getFolderAccess().moveFolder(folderID, newParentFolder.getId(), ignoreWarnings) :
+                getFolderAccess().moveFolder(folderID, newParentFolder.getId(), newName, ignoreWarnings);
         } else if (false == oldName.equals(newName)) {
             /*
              * perform rename only
@@ -1189,11 +1162,22 @@ public class DriveStorage {
         if (session.getTemp().supported() && path.equals(session.getTemp().getPath(false))) {
             return true;
         }
-        if (TypeAware.class.isInstance(folder) && FileStorageFolderType.TRASH_FOLDER.equals(((TypeAware)folder).getType())) {
+        if (isInTrash(folder, path)) {
             return true;
         }
+        return false;
+    }
+
+    /**
+     * Gets a value indicating whether the supplied folder is a trash folder or not.
+     *
+     * @param folder The folder to check
+     * @param path The folder path to check
+     * @return <code>true</code> if the folder is in trash, <code>false</code>, otherwise
+     */
+    public boolean isInTrash(FileStorageFolder folder, String path) throws OXException {
         if (TypeAware.class.isInstance(folder)) {
-            return FileStorageFolderType.TRASH_FOLDER.equals(((TypeAware)folder).getType());
+            return FileStorageFolderType.TRASH_FOLDER.equals(((TypeAware) folder).getType());
         }
         String trashFolderID = hasTrashFolder() && null != getTrashFolder() ? getTrashFolder().getId() : null;
         if (null != trashFolderID && trashFolderID.equals(folder.getId())) {
