@@ -1,55 +1,30 @@
 /*
+ * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @license AGPL-3.0
  *
- *    OPEN-XCHANGE legal information
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *    All intellectual property rights in the Software are protected by
- *    international copyright laws.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Affero General Public License for more details.
  *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with OX App Suite.  If not, see <https://www.gnu.org/licenses/agpl-3.0.txt>.
  *
- *    In some countries OX, OX Open-Xchange, open xchange and OXtender
- *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the OX Software GmbH group of companies.
- *    The use of the Logos is not covered by the GNU General Public License.
- *    Instead, you are allowed to use these Logos according to the terms and
- *    conditions of the Creative Commons License, Version 2.5, Attribution,
- *    Non-commercial, ShareAlike, and the interpretation of the term
- *    Non-commercial applicable to the aforementioned license is published
- *    on the web site http://www.open-xchange.com/EN/legal/index.html.
- *
- *    Please make sure that third-party modules and libraries are used
- *    according to their respective licenses.
- *
- *    Any modifications to this package must retain all copyright notices
- *    of the original copyright holder(s) for the original code used.
- *
- *    After any such modifications, the original and derivative code shall remain
- *    under the copyright of the copyright holder(s) and/or original author(s)per
- *    the Attribution and Assignment Agreement that can be located at
- *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
- *    given Attribution for the derivative code and a license granting use.
- *
- *     Copyright (C) 2016-2020 OX Software GmbH
- *     Mail: info@open-xchange.com
- *
- *
- *     This program is free software; you can redistribute it and/or modify it
- *     under the terms of the GNU General Public License, Version 2 as published
- *     by the Free Software Foundation.
- *
- *     This program is distributed in the hope that it will be useful, but
- *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- *     for more details.
- *
- *     You should have received a copy of the GNU General Public License along
- *     with this program; if not, write to the Free Software Foundation, Inc., 59
- *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Any use of the work other than as authorized under this license or copyright law is prohibited.
  *
  */
 
 package com.openexchange.drive.json.osgi;
 
+import org.osgi.framework.BundleContext;
 import org.osgi.framework.ServiceReference;
+import org.osgi.service.event.EventConstants;
+import org.osgi.service.event.EventHandler;
 import org.osgi.util.tracker.ServiceTrackerCustomizer;
 import com.openexchange.ajax.requesthandler.BodyParser;
 import com.openexchange.ajax.requesthandler.osgiservice.AJAXModuleActivator;
@@ -69,6 +44,7 @@ import com.openexchange.drive.json.internal.UploadActionBodyParser;
 import com.openexchange.drive.json.listener.BlockingListenerFactory;
 import com.openexchange.groupware.notify.hostname.HostnameService;
 import com.openexchange.serverconfig.ServerConfigService;
+import com.openexchange.sessiond.SessiondEventConstants;
 import com.openexchange.share.ShareService;
 import com.openexchange.share.groupware.ModuleSupport;
 import com.openexchange.share.notification.ShareNotificationService;
@@ -101,15 +77,18 @@ public class DriveJsonActivator extends AJAXModuleActivator {
     protected void startBundle() throws Exception {
         LOG.info("starting bundle: \"com.openexchange.drive.json\"");
         Services.set(this);
+        BundleContext context = this.context;
         registerModule(new DriveActionFactory(), "drive");
-        getService(DriveEventService.class).registerPublisher(ListenerRegistrar.getInstance());
+        ListenerRegistrar listenerRegistrar = ListenerRegistrar.getInstance();
+        registerService(EventHandler.class, listenerRegistrar, singletonDictionary(EventConstants.EVENT_TOPIC,
+            new String[] { SessiondEventConstants.TOPIC_REMOVE_SESSION, SessiondEventConstants.TOPIC_REMOVE_CONTAINER }));
+        getService(DriveEventService.class).registerPublisher(listenerRegistrar);
         track(LongPollingListenerFactory.class, new ServiceTrackerCustomizer<LongPollingListenerFactory, LongPollingListenerFactory>() {
 
-            @SuppressWarnings("synthetic-access")
             @Override
             public LongPollingListenerFactory addingService(ServiceReference<LongPollingListenerFactory> serviceReference) {
                 LongPollingListenerFactory service = context.getService(serviceReference);
-                if (ListenerRegistrar.getInstance().addFactory(service)) {
+                if (listenerRegistrar.addFactory(service)) {
                     return service;
                 } else {
                     // already known
@@ -123,11 +102,10 @@ public class DriveJsonActivator extends AJAXModuleActivator {
                 // nothing to do
             }
 
-            @SuppressWarnings("synthetic-access")
             @Override
             public void removedService(ServiceReference<LongPollingListenerFactory> serviceReference, LongPollingListenerFactory service) {
                 try {
-                    ListenerRegistrar.getInstance().removeFactory(service);
+                    listenerRegistrar.removeFactory(service);
                 } finally {
                     context.ungetService(serviceReference);
                 }
